using System;
using System.Globalization;
using System.Windows.Media;
using Utils.Wpf;

namespace StockAnalyst2.Chart
{
	internal class SeriesOptionsVM : ViewModelValidatedBase, IDisposable
	{
		#region Attributes

		private readonly SeriesOptions _model;

		private string _invalidParameterValue;

		#endregion

		#region Properties

		public bool IsEnabled
		{
			get
			{
				return _model.IsEnabled;
			}
			set
			{
				if (_model.IsEnabled != value)
				{
					_model.IsEnabled = value;
					base.OnPropertyChanged("IsEnabled");
				}
			}
		}

		public string Parameter1Value
		{
			get
			{
				return _invalidParameterValue ?? _model.Parameter1Value.ToString(CultureInfo.InvariantCulture);
			}
			set
			{
				int tempValue;
				if (int.TryParse(value, out tempValue))
				{
					if (_model.Parameter1Value != tempValue)
					{
						_model.Parameter1Value = tempValue;
						base.OnPropertyChanged("Parameter1Value");
					}
					_invalidParameterValue = null;
				}
				else
				{
					if (_invalidParameterValue != value)
					{
						_invalidParameterValue = value ?? "";
						base.OnPropertyChanged("Parameter1Value");
					}
				}
			}
		}

		public Brush Color
		{
			get
			{
				return _model.Color;
			}
			set
			{
				if (_model.Color != value)
				{
					_model.Color = value;
					base.OnPropertyChanged("Color");
				}
			}
		}

		public int Thickness
		{
			get
			{
				return _model.Thickness;
			}
			set
			{
				if (_model.Thickness != value)
				{
					_model.Thickness = value;
					base.OnPropertyChanged("Thickness");
				}
			}
		}

		public bool DrawArea
		{
			get
			{
				return _model.DrawArea;
			}
			set
			{
				if (_model.DrawArea != value)
				{
					_model.DrawArea = value;
					base.OnPropertyChanged("DrawArea");
				}
			}
		}

		public Brush AreaColor
		{
			get
			{
				return _model.AreaColor;
			}
			set
			{
				if (_model.AreaColor != value)
				{
					_model.AreaColor = value;
					base.OnPropertyChanged("AreaColor");
				}
			}
		}

		#endregion

		#region Public Methods

		public SeriesOptionsVM(SeriesOptions model)
		{
			_model = model;
			_model.PropertyChanged += Model_PropertyChanged; // Disposed
		}

		protected override string ValidateField(string fieldName)
		{
			string result = null;

			if (fieldName == "Parameter1Value")
			{
				if (_invalidParameterValue != null)
				{
					base.ValidatedErrors[fieldName] = true;
					result = "Please enter an integer";
				}
			}

			return result;
		}

		#endregion

		#region Event Handlers

		private void Model_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
		{
			base.OnPropertyChanged(e.PropertyName);
		}

		#endregion

		#region IDisposable

		protected bool Disposed { get; private set; }

		private void Dispose(bool disposing)
		{
			if (!this.Disposed)
			{
				this.InternalDispose(disposing);
			}

			this.Disposed = true;
		}

		protected virtual void InternalDispose(bool disposing)
		{
			if (disposing)
			{
				_model.PropertyChanged -= this.Model_PropertyChanged;
			}

			// Dispose here only unmanaged objects 
			// Dont use managed objects here because maybe 
			// they have been finalized already
		}

		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		~SeriesOptionsVM()
		{
			this.Dispose(false);
		}

		#endregion
	}
}
