﻿using System;
using Utils.Wpf;
using Visiblox.Charts;

namespace StockAnalyst2.Chart
{
	internal class SeriesVM : ViewModelBase, IDisposable
	{
		#region Attributes

		private readonly Series _model;
		private readonly ModelToVisibloxConverter _converter;

		#endregion

		#region Properties

		public DataSeries<DateTime, decimal> Data { get; private set; }
		public FilteredDataSeries FilteredData { get; private set; }

		#endregion

		#region Public Methods

		public SeriesVM(Series model, string title)
		{
			_model = model;

			this.Data = new DataSeries<DateTime, decimal>(title);

			_converter = new ModelToVisibloxConverter(_model.Data, this.Data, base.Dispatcher);

			this.FilteredData = new FilteredDataSeries();
			this.FilteredData.FilterStrategy = new WindowedFilterStrategy();
			this.FilteredData.OriginalDataSeries = this.Data;
			this.FilteredData.Title = title;
		}

		#endregion

		#region IDisposable

		protected bool Disposed { get; private set; }

		private void Dispose(bool disposing)
		{
			if (!this.Disposed)
			{
				this.InternalDispose(disposing);
			}

			this.Disposed = true;
		}

		protected virtual void InternalDispose(bool disposing)
		{
			if (disposing)
			{
				_converter.Dispose();
			}

			// Dispose here only unmanaged objects 
			// Don’t use managed objects here because maybe 
			// they have been finalized already
		}

		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		~SeriesVM()
		{
			this.Dispose(false);
		}

		#endregion
	}
}
