﻿using System;
using System.Windows;

namespace StockAnalyst2.Options
{
	internal partial class OptionsDialog : IDisposable
	{
		#region Properties

		internal new OptionsVM DataContext
		{
			get
			{
				return (OptionsVM) base.DataContext;
			}
			set
			{
				if (this.DataContext != value)
				{
					if (this.DataContext != null)
					{
						this.DataContext.DialogCloseRequested -= this.Options_DialogCloseRequested;
						this.DataContext.MessageBoxRequested -= this.DataContext_MessageBoxRequested;
					}
					value.DialogCloseRequested += this.Options_DialogCloseRequested; // Disposed
					value.MessageBoxRequested += this.DataContext_MessageBoxRequested; // Disposed
					base.DataContext = value;
				}
			}
		}

		#endregion

		#region Public Methods

		public OptionsDialog()
		{
			InitializeComponent();
			this.UserTextBox.Focus();
		}

		#endregion

		#region Event Handlers

		private void Options_DialogCloseRequested(object sender, Utils.Wpf.DialogCloseRequestedEventArgs e)
		{
			this.DialogResult = e.DialogResult;
			this.Close();
		}

		private void DataContext_MessageBoxRequested(object sender, MessageBoxRequestedEventArgs e)
		{
			MessageBox.Show(this, e.Message, e.Caption, MessageBoxButton.OK, e.Icon);
		}

		#endregion

		#region IDisposable

		protected bool Disposed { get; private set; }

		private void Dispose(bool disposing)
		{
			if (!this.Disposed)
			{
				this.InternalDispose(disposing);
			}

			this.Disposed = true;
		}

		protected virtual void InternalDispose(bool disposing)
		{
			if (disposing)
			{
				if (this.DataContext != null)
				{
					this.DataContext.DialogCloseRequested -= this.Options_DialogCloseRequested;
					this.DataContext.MessageBoxRequested -= this.DataContext_MessageBoxRequested;
				}
			}

			// Dispose here only unmanaged objects 
			// Don’t use managed objects here because maybe 
			// they have been finalized already
		}

		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		~OptionsDialog()
		{
			this.Dispose(false);
		}

		#endregion

	}
}
