﻿using StockModel;
using TechnicalIndicators.Points;
using Utils;

namespace TechnicalIndicators.QuotationBased
{
	public class GapGenerator : GeneratorBase<Quotation, DatedSeriePoint>
	{
		#region Attributes

		private bool _skipPreOpenQuotes;

		#endregion

		#region Public Methods

		public GapGenerator(ObservableCollection2<DatedSeriePoint> destination) : base(destination)
		{
		}

		public void Configure(bool skipPreOpenQuotes)
		{
			_skipPreOpenQuotes = skipPreOpenQuotes;
		}

		protected override DatedSeriePoint CreatePoint(Quotation quotation)
		{
			if (quotation.Sellers[0].Value == 0 || quotation.Buyers[0].Value == 0 || (_skipPreOpenQuotes && quotation.Volume == 0))
			{
				return new DatedSeriePoint(quotation.Date, 0);
			}
			else
			{
				decimal average = (quotation.Sellers[0].Value + quotation.Buyers[0].Value) / 2;
				decimal difference = quotation.Sellers[0].Value - quotation.Buyers[0].Value;
				return new DatedSeriePoint(quotation.Date, difference * 100 / average);
			}
		}

		#endregion
	}
}
